<?php 
define("DUDE_CURL_COMPRESS_GZIP_DEFLATE", "gzip,deflate");
define("DUDE_CURL_COMPRESS_GZIP", "gzip");
define("DUDE_CURL_COMPRESS_DEFLATE", "deflate");
define("DUDE_CURL_COMPRESS_NONE", "");

class DudeCurl
{
	private $_options;
	private $_ch;
	private $_tmpfile;
	private $_cookeijar;
	private $_compression;
	private $_post_encoder;
	private $_content;
	private $_headers;
	private $_cookies;
	private $_referer;
	
	public $responseinfo;
	
	private static $_defaults;
	private static $_defaults_compression = DUDE_CURL_COMPRESS_GZIP_DEFLATE;
	private static $_defaults_post_encoder = "urlencode";
	
	public static $VERSION = "0.2.3";
	public static $VERSIONSTRING = "DudeCurl 0.2.3 Beta";
	
	public function __construct($url = false)
	{
		$this->_check_curl_extension();
		
		$this->init($url);
	}
	
	public function init($url = false)
	{
		$this->_options = array();
		if($url)
		{
			$this->url = $url;
		}
		
		$this->_init_defaults();
		return $this;
	}
	
	public function __set($field, $value)
	{
		$cname = strtoupper("CURLOPT_".trim($field));
		if(!defined($cname))
		{
			throw new Exception("Not found $cname constant. Please check your code");
		}
		
		switch(constant($cname))
		{
			case CURLOPT_HEADER: // This will kick out CURLOPT_HEADER. But we and not nee it.
			case CURLOPT_HTTPHEADER: 
				$value = array_values(!is_array($value) ? explode(":", $value, 2) : $value);
				
				$this->_options["header"][trim($value[0])] = trim($value[1]);
				break;
			default: 
				$this->_options[$field] = $value;
				break;
		}
		
	}
	
	public function opts()
	{
		$vars = func_get_args();
		if(count($vars) == 1)
		{
			foreach (self::toarray($vars[0]) as $opt=>$val)
			{
				$this->$opt = $val;
			}
			
			return $this;
		}
		else if($vars && count($vars) % 2 == 0)
		{
			$vars = array_chunk($vars, 2);
			foreach ($vars as $v)
			{
				$this->$v[0] = $v[1];
			}
			
			return $this;
		}
		else if(!$count)
		{
			return $this->_options;
		}
		
		throw new Exception('Bad parameters count given for opts($mixed, param $mixed $_, ...) function');
	}
	
	public function compression($comression)
	{
		$this->_compression = $comression;
	}
	
	public function post_encoder($encoder)
	{
		$this->_post_encoder = $encoder;
	}
	
	public function header($name, $last = true)
	{
		return !$last ? (array)@$this->_headers[strtolower($name)] : @array_pop($this->_headers[strtolower($name)]);
	}
	
	public function headers()
	{
		return $this->_headers;
	}
	
	public function addCookie($name, $value = null, $expires = null, $path = null, $domain = null)
	{
		$this->_cookies[$domain][$path][$name] = array_filter(array("expiresint"=>$expires, "path"=>$path, "domain"=>$domain, "value"=>$value));
		return $this;
	}
	
	public function cookies($url = "", $name = false, $valueonly = true)
	{
		$cookies = array();
		if(!$url && !$name)
		{
			$cookies = $this->_cookies;
		}
		else
		{
			$urlinfo = parse_url($url);
			$all = $this->_cookies;
			$cookies = array();
			foreach ((array)$all as $domain=>$cooks)
			{
				$domain = $domain ? $domain : parse_url($this->_options['url'],PHP_URL_HOST);
				$domain = ltrim($domain, ".");
				if($urlinfo['host'] && substr($urlinfo['host'], -strlen($domain)) != $domain)
				{
					continue;
				}
				foreach ((array)$cooks as $curl=>$coooks)
				{
					if(!$urlinfo['path'] || substr($urlinfo['path'], 0, strlen($curl)) == $curl)
					{
						$cookies[] = $coooks;
					}
				}
			}
			
			$cookies = $cookies ? call_user_func_array("array_merge", $cookies) : array();
		}
		if($valueonly)
		{
			foreach ($cookies as $n=>$data)
			{
				$cookies[$n] = $data['value'];
			}
		}
		
		return $name ? $cookies[$name] : $cookies;
	}
	
	public function addCookies($cookies)
	{
		foreach ((array)$cookies as $name=>$cookie)
		{
			@$this->addCookie($name, $cookie['value'], max(strtotime($cookie['expires']), $cookie['expiresint']), $cookie['path'], $cookie['domain']);
		}
	}
	
	public function content()
	{
		return $this->_content;
	}
	
	public function exec($return = true)
	{
		$err = error_reporting(E_ALL & ~E_NOTICE);
		
		$this->exec_pre($return);
		$return = $this->exec_post(curl_exec($this->_ch));
		
		error_reporting($err);
		
		return $return;
	}
	
	public function exec_pre($return)
	{
		if(!$this->_options['url'])
		{
			throw new Exception("Not set URL option. Please check your code");
		}
		
		$this->_ch = curl_init();
		$this->_tmpfile = tmpfile();
		$this->_cookeijar = tempnam(dirname(__FILE__), "cookiejar_");
		
		curl_setopt($this->_ch, CURLOPT_WRITEHEADER, $this->_tmpfile);
		curl_setopt($this->_ch, CURLOPT_RETURNTRANSFER, $return ? 1 : 0);
		curl_setopt($this->_ch, CURLOPT_COOKIESESSION, 1);
	    curl_setopt($this->_ch, CURLOPT_USERAGENT, 'Opera/9.80 (J2ME/MIDP; Opera Mini/4.2.14912/1280; U; ru) Presto/2.2.0');
		curl_setopt($this->_ch, CURLOPT_COOKIEFILE, $this->_cookeijar);
		curl_setopt($this->_ch, CURLOPT_COOKIEJAR, $this->_cookeijar);
		curl_setopt($this->_ch, CURLOPT_HEADER, 0);
		
		if(!ini_get("open_basedir") && !ini_get("safe_mode"))
		{
			curl_setopt($this->_ch, CURLOPT_FOLLOWLOCATION, 1);
		}
		if($this->_referer)
		{
			curl_setopt($this->_ch, CURLOPT_REFERER, $this->_referer);
		}
		
		$compressions = array();
		$compressions[] = DUDE_CURL_COMPRESS_GZIP_DEFLATE;
		$compressions[] = DUDE_CURL_COMPRESS_GZIP;
		$compressions[] = DUDE_CURL_COMPRESS_DEFLATE;
		$compressions[] = DUDE_CURL_COMPRESS_NONE;
		if($this->_compression != "" && in_array($this->_compression, $compressions))
		{
			$this->_options['encoding'] = $this->_compression;
			$this->_options['header']['Accept-Encoding'] = $this->_compression;
		}
		
		foreach ($this->_options as $field=>$value)
		{
			switch($field)
			{
				case "header": 
					foreach ($value as $header=>$val)
					{
						$headers[] = ucfirst(strtolower($header)).": $val";
					}
					curl_setopt($this->_ch, CURLOPT_HTTPHEADER, $headers);
					break;
					
				case "post": 
					if(is_array($value) && !preg_grep("#^@#", $value))
					{
						$j = array();
						foreach ($value as $k=>$v)
						{
							$j[] = call_user_func($this->_post_encoder, $k)."=".call_user_func($this->_post_encoder, $v);
						}
						$value = join("&", $j);
					}
					curl_setopt($this->_ch, CURLOPT_POST, 1);
					curl_setopt($this->_ch, CURLOPT_POSTFIELDS, $value);
					break;
					
				default: 
					curl_setopt($this->_ch, constant(strtoupper("CURLOPT_".$field)), $value);
					break;
			} 
		}
		
		if($cookies = $this->cookies($this->_options['url']))
		{
			$cookies = http_build_query($cookies, "", ";");
			curl_setopt($this->_ch, CURLOPT_COOKIE, $cookies);
		}
		
		return $this;
	}
	
	public function exec_post($return)
	{
		if(curl_errno($this->_ch))
		{
			@unlink($this->_cookeijar);
			$errno = curl_errno($this->_ch);
			$error = curl_error($this->_ch);
			curl_close($this->_ch);
			throw new Exception("CURL ERROR: ".$error, $errno);
		}
		
		$this->responseinfo = new DudeCurlResponseInfo();
		foreach (curl_getinfo($this->_ch) as $f=>$v)
		{
			$this->responseinfo->$f = $v;
		}
		$this->_referer = $this->responseinfo->url;
		
		curl_close($this->_ch);
		@unlink($this->_cookeijar);
		
		$this->_content = $return;
		
		$this->_header_parser();
		$this->_toutf();
		
		return $this->_content;
	}
	
	public static function set_defaults($options, $compression = DUDE_CURL_COMPRESS_GZIP_DEFLATE , $post_encoder = "urlencode")
	{
		foreach (self::toarray($options) as $field=>$value)
		{
			self::$_defaults[$field] = $value;
		}
		
		self::$_defaults_compression = $compression;
		
		if(!function_exists($post_encoder))
		{
			throw new Exception("Function $post_encoder() doesn't exists");
		}
		self::$_defaults_post_encoder = $post_encoder;
	}
	
	public static function get_defaults()
	{
		return array((array)self::$_defaults, self::$_defaults_compression);
	}
	
	public static function toarray($options)
	{
		if(is_object($options))
		{
			$options = get_object_vars($options);
		}
		
		if(!is_array($options))
		{
			throw new Exception("set_defaults function must be called with object or array param");
		}
		
		return $options;
	}
	
	private function _check_curl_extension()
	{
		if(!extension_loaded('curl'))
		{
			if(strtoupper(substr(PHP_OS, 0, 3)) === 'WIN')
			{
				@dl('curl.dll');
			}
			else
			{
				@dl('curl.so');
			}
		}
		if(!extension_loaded('curl'))
		{
			throw new Exception("Cannot load Curl extesion.");
		}
	}
	
	private function _init_defaults()
	{
		if(is_array(self::$_defaults))
		{
			foreach (self::$_defaults as $opt=>$val)
			{
				switch($opt)
				{
					case "header": 
						$val = (is_array($val) ? $val : array($val));
						foreach ($val as $header)
						{
							$this->header = $header;
						}
						break;
						
					default: 
						$this->$opt = $val;
						break;
				} 
			}
		}
		
		$this->_compression = self::$_defaults_compression;
		$this->_post_encoder = self::$_defaults_post_encoder;
		
		$this->header = "Expect: "; // stupid HTTP/1.0 bug feature fix.
	}
	
	private function _header_parser()
	{
		$tmpfile = $this->_tmpfile;
		
		fseek($tmpfile, 0);
		fgets($tmpfile);
		
		while(!feof($tmpfile) && is_resource($tmpfile))
		{
			$line = trim(fgets($tmpfile));
			if(!$pos = strpos($line, ":"))
			{
				continue;
			}
			
			$header = strtolower(trim((substr($line, 0, $pos))));
			$val = trim(substr($line, $pos + 1));
			$this->_headers[$header][] = $val;
		}
		
		$this->_cookies_parser();
	}
	
	private function _cookies_parser()
	{
		$defdomain = preg_replace("#^www\\.#","",parse_url($this->_options['url'],PHP_URL_HOST));
		$cookies = $this->header("Set-Cookie", false);
		if(!$cookies)
		{
			return;
		}
		if(!is_array($cookies))
		{
			$cookies = array($cookies);
		}
		foreach ($cookies as $cookie)
		{
			$cookiename = "";
			$cookieinfo = array();
			$data = explode(";", $cookie);
			foreach ($data as $part)
			{
				$parts = explode("=", $part, 2);
				if(!$cookiename)
				{
					$cookiename = $parts[0];
					$cookieinfo = array("name"=>$cookiename, "value"=>rawurldecode($parts[1]));
				}
				else
				{
					$cookieinfo[strtolower(trim(($parts[0])))] = $parts[1];
				}
			}
			if($cookieinfo['expires'])
			{
				$cookieinfo['expiresint'] = strtotime($cookieinfo['expires']);
			}

			$cookieinfo['domain'] = $cookieinfo['domain'] ? $cookieinfo['domain'] : $defdomain;

			$this->_cookies[$cookieinfo['domain']][$cookieinfo['path']][$cookiename] = $cookieinfo;
		}
	}
	
	private function _toutf()
	{
		if($contenttype = $this->responseinfo->content_type)
		{
			if(preg_match('#text/html\s*;\s*charset\s*=\s*([^\s]+)#i', $contenttype, $charset))
			{
				$charset = $charset[1];
			}
		}
		if(preg_match("#<meta[^>]+Content.Type[^>]+>#ism", $this->_content, $contenttype))
		{
			if(preg_match('#\s*;\s*charset\s*=\s*(["\'a-z0-9\\-]+)#i', $contenttype[0], $chset))
			{
				$charset = trim($chset[1], "\"'");
			}
		}
		if($charset && strtolower($charset) != "utf-8")
		{
			$this->_content = iconv($charset, "UTF-8", $this->_content);
		}
	} 
}

class DudeCurlResponseInfo
{
	public $url;
	public $content_type;
	public $http_code;
	public $header_size;
	public $request_size;
	public $filetime;
	public $ssl_verify_result;
	public $redirect_count;
	public $total_time;
	public $namelookup_time;
	public $connect_time;
	public $pretransfer_time;
	public $size_upload;
	public $size_download;
	public $speed_download;
	public $speed_upload;
	public $download_content_length;
	public $upload_content_length;
	public $starttransfer_time;
	public $redirect_time;
}
